import factory

from django.contrib.auth import get_user_model


class UserFactory(factory.django.DjangoModelFactory):
    username = factory.Sequence(lambda n: "user-{0}".format(n))
    email = factory.Sequence(lambda n: "user-{0}@example.com".format(n))
    # setting default password to "password", but you can also pass different ones
    password = factory.PostGenerationMethodCall("set_password", "password")

    class Meta:
        model = get_user_model()
        django_get_or_create = ("username",)



import os
import io
import pytest

from django.conf import settings
from django.core.files.storage import default_storage
from django.core.files.uploadedfile import SimpleUploadedFile

from .factories import UserFactory
from filepond.utils import storage_walk_paths


@pytest.fixture(scope="session", autouse=True)
def media_dir_cleanup():
    # Will be executed before the first test
    # and collects all paths in media root, that already
    # existed before running the tests
    print("media root: ", settings.MEDIA_ROOT)
    existing_paths = set(storage_walk_paths(default_storage))
    yield existing_paths
    # Will be executed after the last test
    # and finds all paths in media root which were added
    # during the tests and remove them
    for path in storage_walk_paths(default_storage):
        if path not in existing_paths:
            abs_path = os.path.join(settings.MEDIA_ROOT, path)
            os.unlink(abs_path)


@pytest.fixture()
def fixture_dir():
    curdir = os.path.dirname(os.path.realpath(__file__))
    return os.path.join(curdir, "fixtures")


@pytest.fixture()
def default_password():
    return "password"


@pytest.fixture()
def user(default_password):
    user = UserFactory(password=default_password)
    user._password = default_password
    return user


@pytest.fixture()
def user2(default_password):
    user = UserFactory(password=default_password)
    user._password = default_password
    return user


def create_small_rgb():
    # this is a small test jpeg
    from PIL import Image

    img = Image.new("RGB", (200, 200), (255, 0, 0, 0))
    return img


@pytest.fixture()
def small_jpeg_io():
    rgb = create_small_rgb()
    im_io = io.BytesIO()
    rgb.save(im_io, format="JPEG", quality=60, optimize=True, progressive=True)
    im_io.seek(0)
    im_io.name = "testimage.jpg"
    return im_io


def read_test_m4a(fixture_dir):
    with open(os.path.join(fixture_dir, "test.m4a"), "rb") as f:
        m4a = f.read()
    return m4a


@pytest.fixture()
def m4a_audio_django_file(fixture_dir):
    m4a = read_test_m4a(fixture_dir)
    simple_m4a = SimpleUploadedFile(
        name="test.m4a", content=m4a, content_type="audio/mp4"
    )
    return simple_m4a


@pytest.fixture()
def m4a_audio_io(fixture_dir):
    m4a = read_test_m4a(fixture_dir)
    audio_io = io.BytesIO(m4a)
    audio_io.seek(0)
    audio_io.name = "testaudio.m4a"
    return audio_io



from django.urls import path, re_path
from django.views.decorators.csrf import csrf_exempt

from . import views

app_name = "filepond"
urlpatterns = [
    path("process", view=csrf_exempt(views.UploadCreateView.as_view()), name="process"),
    path(
        # process, but give an app name to dispatch the upload to
        "process/<path:app_name>",
        view=csrf_exempt(views.UploadCreateView.as_view()),
        name="process",
    ),
    path(
        "revert/",
        view=csrf_exempt(views.UploadRevertView.as_view()),
        name="revert",
    ),
    re_path(
        r"^get/(?P<pk>[0-9]+)/(?P<size>[\w]+)/$",
        view=views.FileView.as_view(),
        name="get",
    ),
]



# Generated by Django 2.1.3 on 2018-11-27 15:38

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
        ("filepond", "0002_auto_20181127_0925"),
    ]

    operations = [
        migrations.AddField(
            model_name="upload",
            name="content_type",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="contenttypes.ContentType",
            ),
        ),
        migrations.AddField(
            model_name="upload",
            name="object_id",
            field=models.PositiveIntegerField(blank=True, null=True),
        ),
    ]



import pytest

from django.urls import reverse

from filepond.models import Upload
from testapp.models import LocalAudio


class TestAudioUpload:
    upload_url = reverse("filepond:process")
    upload_to_testapp_url = reverse("filepond:process", kwargs={"app_name": "testapp"})
    revert_url = reverse("filepond:revert")

    @pytest.mark.django_db
    def test_upload_m4a_audio_with_app(self, client, user, m4a_audio_io):
        # login
        r = client.login(username=user.username, password=user._password)

        # upload
        m4a_audio_io.seek(0)
        r = client.post(self.upload_to_testapp_url, {"filepond": m4a_audio_io})
        upload_pk = int(r.content.decode("utf-8"))

        assert r.status_code == 201
        assert upload_pk > 0

        upload = Upload.objects.get(pk=upload_pk)
        audio = upload.content_object

        assert audio.__class__ == LocalAudio
        assert audio.m4a.path == upload.original.path



from django import forms

from .models import Upload


class UploadForm(forms.ModelForm):
    class Meta:
        model = Upload
        fields = ["original"]


def get_model_form(
    upload_model, upload_field_name="original", upload_fields=None, user_field=None
):
    """Convenience function to Create an upload form."""
    if upload_fields is None:
        upload_fields = []
    if upload_field_name not in upload_fields:
        upload_fields.append(upload_field_name)

    class DynamicUploadForm(forms.ModelForm):
        class Meta:
            model = upload_model
            fields = upload_fields

    context = {"user_field": user_field, "upload_field_name": upload_field_name}
    return DynamicUploadForm, context



from django.apps import apps
from django.contrib.auth.mixins import LoginRequiredMixin
from django.http import HttpResponse
from django.views.generic import CreateView, DeleteView, DetailView

from .forms import get_model_form
from .models import Upload


def default_upload_handler(request):
    """
    Default upload handler saves files to uploads dir and
    return no other Model that should be notified about
    upload.
    """

    def get_upload_path(instance, filename):
        return f"uploads/{filename}"

    return get_upload_path, None


class UploadCreateView(LoginRequiredMixin, CreateView):
    def get_success_url(self):
        return None

    def get_form_kwargs(self):
        """If an instance is in context, add it to form creation and
        switch from create to update."""
        form_kwargs = super().get_form_kwargs()
        if "instance" in self.context:
            form_kwargs["instance"] = self.context["instance"]
        return form_kwargs

    def form_valid(self, form):
        user_field = self.context["user_field"]
        upload_field_name = self.context["upload_field_name"]

        # save upload into target model
        model = form.save(commit=False)
        if user_field is not None:
            setattr(model, user_field, self.request.user)
        super().form_valid(form)

        # if target model != Upload, save an Upload instance, too
        upload_pk = model.pk
        if model.__class__ != Upload:
            upload = Upload.objects.create(
                original=getattr(model, upload_field_name),
                user=self.request.user,
                content_object=model,
            )
            upload_pk = upload.pk

        # return primary key of upload model, because we don't want
        # to have to guess which model to delete on post to revert endpoint
        return HttpResponse(f"{upload_pk}", status=201)

    def form_invalid(self, form):
        return super().form_invalid(form)

    def register_upload_app(self, request, *args, **kwargs):
        app_name = kwargs.get("app_name")
        if app_name is None:
            # if there's no app specified, use own model
            self.form_class, self.context = get_model_form(Upload, user_field="user")
            return

        # get the new form_class
        upload_handler = apps.get_app_config(app_name).module.upload_handler
        self.form_class, self.context = upload_handler(request)

        # move file input label to requested upload_field_name
        upload_field_name = self.context["upload_field_name"]
        if upload_field_name != "original":
            request.FILES[upload_field_name] = request.FILES.pop("original")[0]

    def post(self, request, *args, **kwargs):
        # if file was uploaded by filepond, change the field name
        if "filepond" in request.FILES:
            request.FILES["original"] = request.FILES.pop("filepond")[0]
        self.register_upload_app(request, *args, **kwargs)
        return super().post(request, *args, **kwargs)


class UploadRevertView(LoginRequiredMixin, DeleteView):
    model = Upload

    def post(self, request, *args, **kwargs):
        upload = self.model.objects.get(pk=int(request.body))
        if self.request.user == upload.user:
            if upload.content_object is not None:
                upload.content_object.delete()
            upload.delete()
        return HttpResponse("", status=200)


class FileView(LoginRequiredMixin, DetailView):
    model = Upload



# -*- coding: utf-8
from __future__ import unicode_literals, absolute_import

import environ

ROOT_DIR = environ.Path(__file__) - 1
APPS_DIR = ROOT_DIR.path("filepond")

DEBUG = True
USE_TZ = True

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = "lbk&8(j#cf#=22u!09m9vx$@ca+8ar^oj^#q&k+*v*_4zio_2&"

DATABASES = {"default": {"ENGINE": "django.db.backends.sqlite3", "NAME": ":memory:"}}

ROOT_URLCONF = "tests.urls"

INSTALLED_APPS = [
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sites",
    "django.contrib.sessions",
    "filepond",
    "testapp",
    # "filepond.apps.FilepondConfig",
    # "testapp.apps.TestappConfig",
]

SITE_ID = 1

MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
]

TEST_RUNNER = "filepond.runner.PytestTestRunner"

# MEDIA CONFIGURATION
# ------------------------------------------------------------------------------
# See: https://docs.djangoproject.com/en/dev/ref/settings/#media-root
MEDIA_ROOT = str(ROOT_DIR("media"))

# See: https://docs.djangoproject.com/en/dev/ref/settings/#media-url
MEDIA_URL = "/media/# MEDIA CONFIGURATION"
# ------------------------------------------------------------------------------
# See: https://docs.djangoproject.com/en/dev/ref/settings/#media-root
MEDIA_ROOT = str(ROOT_DIR("media"))

# See: https://docs.djangoproject.com/en/dev/ref/settings/#media-url
MEDIA_URL = "/media/"

# TEMPLATE CONFIGURATION
# ------------------------------------------------------------------------------
# See: https://docs.djangoproject.com/en/dev/ref/settings/#templates
TEMPLATES = [
    {
        # See: https://docs.djangoproject.com/en/dev/ref/settings/#std:setting-TEMPLATES-BACKEND
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        # See: https://docs.djangoproject.com/en/dev/ref/settings/#template-dirs
        "DIRS": [str(APPS_DIR.path("templates"))],
        "OPTIONS": {
            # See: https://docs.djangoproject.com/en/dev/ref/settings/#template-debug
            "debug": DEBUG,
            # See: https://docs.djangoproject.com/en/dev/ref/settings/#template-loaders
            # https://docs.djangoproject.com/en/dev/ref/templates/api/#loader-types
            "loaders": [
                "django.template.loaders.filesystem.Loader",
                "django.template.loaders.app_directories.Loader",
            ],
            # See: https://docs.djangoproject.com/en/dev/ref/settings/#template-context-processors
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.template.context_processors.i18n",
                "django.template.context_processors.media",
                "django.template.context_processors.static",
                "django.template.context_processors.tz",
                "django.contrib.messages.context_processors.messages",
                # Your stuff: custom template context processors go here
            ],
        },
    }
]

# TEMPLATE LOADERS
# ------------------------------------------------------------------------------
# Keep templates in memory so tests run faster
TEMPLATES[0]["OPTIONS"]["loaders"] = [
    [
        "django.template.loaders.cached.Loader",
        [
            "django.template.loaders.filesystem.Loader",
            "django.template.loaders.app_directories.Loader",
        ],
    ]
]



def upload_handler(request):
    from . import models
    from filepond.forms import get_model_form

    lookup = {}
    for ending in ("jpg", "jpeg", "png", "gif"):
        lookup[ending] = (models.LocalImage, "image", "owner")

    for ending in ("m4a", "mp3"):
        lookup[ending] = (models.LocalAudio, ending, "owner")

    file_name = str(request.FILES["original"])
    ending = file_name.lower().split(".")[-1]
    print("request filename: ", file_name, ending)
    local_model, upload_field_name, user_field = lookup[ending]
    return get_model_form(
        local_model, upload_field_name=upload_field_name, user_field=user_field
    )



from django.conf import settings
from rest_framework_jwt.authentication import JSONWebTokenAuthentication

JWT_COOKIE_NAME = settings.JWT_AUTH.get("JWT_AUTH_COOKIE")


class JWTMiddleware:
    """
    Enable djangorestframework-jwt authentication also for non
    rest framework requests like normal django views and graphql
    requests.
    """

    def __init__(self, get_response):
        self.get_response = get_response

    def process_view(self, request, view_func, view_args, view_kwargs):
        # token should be either in http headers or cookie..
        token = request.META.get("HTTP_AUTHORIZATION", "")
        if not token.startswith("JWT") and JWT_COOKIE_NAME not in request.COOKIES:
            return
        jwt_auth = JSONWebTokenAuthentication()
        auth = None
        try:
            auth = jwt_auth.authenticate(request)
        except Exception:
            return
        if auth is not None:
            request.user = auth[0]

    def __call__(self, request):
        return self.get_response(request)



from django.db import models
from django.urls import reverse
from django.contrib.auth import get_user_model
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes.fields import GenericForeignKey


from model_utils.models import TimeStampedModel


class Upload(TimeStampedModel):
    """
    Hold uploaded files and a generic relation to the model
    that has beed registered for this upload.
    """

    # request.user and uploaded file
    user = models.ForeignKey(get_user_model(), on_delete=models.CASCADE)
    original = models.FileField(upload_to="uploads")

    # generic foreign key
    content_type = models.ForeignKey(
        ContentType, on_delete=models.CASCADE, null=True, blank=True
    )
    object_id = models.PositiveIntegerField(null=True, blank=True)
    content_object = GenericForeignKey("content_type", "object_id")

    @property
    def original_url(self):
        return reverse("filepond:get", kwargs={"pk": self.pk, "size": "original"})

    def __str__(self):
        return f"{self.pk} {self.original.name}"



# Generated by Django 2.1.3 on 2018-11-27 15:25

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("filepond", "0001_initial")]

    operations = [
        migrations.AlterField(
            model_name="upload",
            name="original",
            field=models.FileField(upload_to="uploads"),
        )
    ]



import pytest

from django.urls import reverse

from filepond.models import Upload
from testapp.models import LocalImage


class TestImageUpload:
    upload_url = reverse("filepond:process")
    upload_to_testapp_url = reverse("filepond:process", kwargs={"app_name": "testapp"})
    revert_url = reverse("filepond:revert")

    @pytest.mark.django_db
    def test_upload_image_not_authenticated(self, client, small_jpeg_io):
        small_jpeg_io.seek(0)
        r = client.post(self.upload_url, {"filepond": small_jpeg_io})
        # redirect to login
        assert r.status_code == 302

    @pytest.mark.django_db
    def test_upload_image_authenticated(self, client, user, small_jpeg_io):
        # login
        r = client.login(username=user.username, password=user._password)

        # upload
        small_jpeg_io.seek(0)
        r = client.post(self.upload_url, {"filepond": small_jpeg_io})

        assert r.status_code == 201
        assert int(r.content.decode("utf-8")) > 0

    @pytest.mark.django_db
    def test_revert_image_upload_wrong_user(self, client, user, user2, small_jpeg_io):
        assert user.pk != user2.pk
        # login
        r = client.login(username=user.username, password=user._password)

        # upload
        small_jpeg_io.seek(0)
        r = client.post(self.upload_url, {"filepond": small_jpeg_io})
        assert r.status_code == 201

        # revert with wrong user
        image_pk = int(r.content.decode("utf-8"))
        assert Upload.objects.filter(pk=image_pk).count() == 1

        client.logout()
        r = client.login(username=user2.username, password=user2._password)

        r = client.post(self.revert_url, str(image_pk), content_type="application/json")
        assert r.status_code == 200
        assert Upload.objects.filter(pk=image_pk).count() == 1

    @pytest.mark.django_db
    def test_revert_image_upload(self, client, user, small_jpeg_io):
        # login
        r = client.login(username=user.username, password=user._password)

        # upload
        small_jpeg_io.seek(0)
        r = client.post(self.upload_url, {"filepond": small_jpeg_io})
        assert r.status_code == 201

        # revert
        image_pk = int(r.content.decode("utf-8"))
        assert Upload.objects.filter(pk=image_pk).count() == 1

        r = client.post(self.revert_url, str(image_pk), content_type="application/json")
        assert r.status_code == 200
        assert Upload.objects.filter(pk=image_pk).count() == 0

    @pytest.mark.django_db
    def test_upload_image_with_app(self, client, user, small_jpeg_io):
        # login
        r = client.login(username=user.username, password=user._password)

        # upload
        small_jpeg_io.seek(0)
        r = client.post(self.upload_to_testapp_url, {"filepond": small_jpeg_io})
        upload_pk = int(r.content.decode("utf-8"))

        assert r.status_code == 201
        assert upload_pk > 0

        upload = Upload.objects.get(pk=upload_pk)
        image = upload.content_object
        assert image.__class__ == LocalImage
        assert image.image.path == upload.original.path



from django.db import models
from django.contrib.auth import get_user_model

from model_utils.models import TimeStampedModel


class LocalImage(TimeStampedModel):
    """
    Receive test images via filepond.
    """

    owner = models.ForeignKey(get_user_model(), on_delete=models.CASCADE)
    image = models.ImageField(upload_to="testapp/images")

    def __str__(self):
        return f"{self.pk} {self.image.name}"


class LocalAudio(TimeStampedModel):
    """
    Receive test audio tracks via filepond.
    """

    owner = models.ForeignKey(get_user_model(), on_delete=models.CASCADE)
    m4a = models.FileField(upload_to="testapp/audio", blank=True, null=True)
    mp3 = models.FileField(upload_to="testapp/audio", blank=True, null=True)

    def __str__(self):
        return f"audio: {self.pk}"
